/**********
Copyright 1990 Regents of the University of California.  All rights reserved.
Author: 1985 Wayne A. Christopher
**********/

/*
 * The main routine for ngspice
 */


#include "spice.h"
#include "ifsim.h"
#include "inpdefs.h"
#include "iferrmsg.h"
#include "cpdefs.h"
#include "ftedefs.h"
#include "ftedev.h"
#include "ftedebug.h"
#include "const.h"
#include <setjmp.h>
#include <signal.h>

#include <sys/types.h>

#ifdef HAVE_PWD_H
#include <pwd.h>
#endif



#ifndef HAVE_GETRUSAGE
#ifdef HAVE_FTIME
#include <sys/timeb.h>
#endif
#endif



/* (Virtual) Machine architecture parameters */
int ARCHme;
int ARCHsize;

char *ft_rawfile = "rawspice.raw";

char *errRtn;
char *errMsg;
char *cp_program;

bool ft_servermode = false;
bool ft_intrpt = false;     /* Set by the (void) signal handlers. */
bool ft_setflag = false;    /* Don't abort after an interrupt. */

struct variable *(*if_getparam)( );



bool ft_batchmode = false;
jmp_buf jbuf;
static char *usage = 
"Usage: %s [-] [-b] [-i] [-s] [-n] [-o outfile] [-r rawfile]\n\
\t[-t term] [file ...]\n";
struct options *exitoption;
struct options *helpoption;


static int started = false;
IFsimulator *ft_sim = 0;



#ifndef HAVE_GETRUSAGE
#ifdef HAVE_FTIME
extern struct timeb timebegin;		/* for use w/ ftime */
#endif
#endif

extern IFsimulator SIMinfo;

#ifdef SIMULATOR

bool ft_nutmeg = false;


extern struct comm spcp_coms[ ];
struct comm *cp_coms = spcp_coms;

extern int OUTpBeginPlot(), OUTpData(), OUTwBeginPlot(), OUTwReference();
extern int OUTwData(), OUTwEnd(), OUTendPlot(), OUTbeginDomain();
extern int OUTendDomain(), OUTstopnow(), OUTerror(), OUTattributes();
static IFfrontEnd nutmeginfo = {
    IFnewUid,
    IFdelUid,
    OUTstopnow,
    seconds,
    OUTerror,
    OUTpBeginPlot,
    OUTpData,
    OUTwBeginPlot,
    OUTwReference,
    OUTwData,
    OUTwEnd,
    OUTendPlot,
    OUTbeginDomain,
    OUTendDomain,
    OUTattributes
};

#else

bool ft_nutmeg = true;
extern struct comm nutcp_coms[ ];
struct comm *cp_coms = nutcp_coms;
static IFfrontEnd nutmeginfo;
/* XXX */
/* ARGSUSED */ int if_run(t, w, s, b) char *t, *w, *b; wordlist *s;
        { return (0); }
/* ARGSUSED */ int if_sens_run(t, w, s, b) char *t, *w, *b; wordlist *s;
        { return (0); }
/* ARGSUSED */ void if_dump(ckt, fp) char *ckt; FILE *fp; { }
/* ARGSUSED */ char * if_inpdeck(deck, tab) struct line *deck; char **tab;
        { return ((char *) 0); }
/* ARGSUSED */ int if_option(ckt, name, type, value) char *name, *ckt, *value;
        int type; { }
/* ARGSUSED */ void if_cktfree(ckt, tab) char *ckt, *tab; { }
/* ARGSUSED */ void if_setndnames(line) char *line; { }
/* ARGSUSED */ char * if_errstring(code) { return ("spice error"); }
/* ARGSUSED */ void if_setparam(ckt, name, param, val) 
        char *ckt, *name, *param;  struct variable *val; { }
/* ARGSUSED */
bool
if_tranparams(ckt, start, stop, step) struct circ *ckt;
        double *start, *stop, *step;
{ return (false); }
/* ARGSUSED */ struct variable *if_getstat(n, c) char *n, *c; { return (NULL);}

#endif /* SIMULATOR */

char *hlp_filelist[] = { "ngspice", 0 };

int
main(ac, av)
    char **av;
{
    char	**tv;
    int		tc;
    int		err;
    bool	gotone = false;
    char	*cmd_line_term = 0, term_1stch;

#ifdef SIMULATOR

    int		i,error2;

#endif /* SIMULATOR */


#ifdef HAVE_PWD_H
    struct passwd *pw;
#endif
    char	buf[BSIZE_SP];
    bool	readinit = true, rflag = false, ciprefix();
    bool	istty = true, iflag = false, qflag = false;
    bool	gdata = true;
    FILE	*fp;
    FILE	*circuit_file;


    /* MFB tends to jump to 0 on errors.  This tends to catch it. */
    if (started) {
        fprintf(cp_err, "main: Internal Error: jump to zero\n");
        shutdown(EXIT_BAD);
    }
    started = true;

#ifdef PARALLEL_ARCH
    PBEGIN_(ac, av);
    ARCHme = NODEID_();
    ARCHsize = NNODES_();
    SETDBG_(&debug_flag);
    fprintf( stderr, "On-line: process %d of %d total.\n", ARCHme, ARCHsize );
    evlog(EVKEY_ENABLE, EVKEY_EVENT, "On-line", EVKEY_DUMP, EVKEY_DISABLE,
	    EVKEY_LAST_ARG);
#else
    ARCHme = 0;
    ARCHsize = 1;
#endif /* PARALLEL_ARCH */


    ivars( );

    cp_in = stdin;
    cp_out = stdout;
    cp_err = stderr;

    circuit_file = stdin;



#ifdef MALLOCTRACE
    mallocTraceInit("malloc.out");
#endif
#ifdef HAVE_ISATTY
    istty = (bool) isatty(fileno(stdin));
#endif



    init_time( );

    err = SIMinit(&nutmeginfo,&ft_sim);
    if(err != OK) {
        ft_sperror(err,"SIMinit");
        shutdown(EXIT_BAD);
    }
    cp_program = ft_sim->simulator;

    srandom(getpid());


#ifdef PARALLEL_ARCH
#ifdef NOTDEF
    tv = av;
    tc = ac;

    /* Print Arguments. */
    fprintf( stderr, "\nArgs:\n" );
    while (--tc > 0) {
        tv++;
	fprintf( stderr, "\t%s\n", *tv );
    }
#endif /* NOTDEF */

    /* Skip processor group file. */ /* XXX What if it's not first? */
    ac--; av++;
#endif /* PARALLEL_ARCH */

    tv = av;
    tc = ac;

    /* Pass 1 -- get options. */
    while (--tc > 0) {
        tv++;
        if (**tv == Spice_OptChar)    /* Option argument. */
            switch ((*tv)[1]) {


	    case '\0':  /* No raw file. */
                gdata = false;
                break;

	    case 'b':   /* Batch mode. */
	    case 'B':
                ft_batchmode = true;
                break;

	    case 's':   /* Server mode. */
	    case 'S':
                ft_servermode = true;
                break;

	    case 'i':   /* Interactive mode. */
	    case 'I':
                iflag = true;
                break;

	    case 'q':   /* No command completion. */
	    case 'Q':
                qflag = true;
                break;

	    case 'n':   /* Don't read .spiceinit. */
	    case 'N':
                readinit = false;
                break;

	    case 't':   /* Terminal type. */
	    case 'T':
                if (tc > 1) {
                    tc--;
                    tv++;
		    cmd_line_term = *tv;
		    term_1stch = **tv;
                    **tv = Spice_OptChar;
                } else {
                    fprintf(cp_err, usage, 
                        cp_program);
                    shutdown(EXIT_BAD);
                }
                break;

	    case 'r':   /* The rawfile. */
	    case 'R':
                if (tc > 1) {
                    tc--;
                    tv++;
		    sprintf( buf, "%s", *tv );
		    cp_vset("rawfile", VT_STRING, buf);
                    **tv = Spice_OptChar;
                }
                rflag = true;
                break;

	    case 'o':   /* Output file. */
	    case 'O':
                if (tc > 1) {
                    tc--;
                    tv++;
#ifdef PARALLEL_ARCH
		    sprintf( buf, "%s%03d", *tv, ARCHme );
#else
		    sprintf( buf,"%s",*tv);
/*		    sprintf( buf, "%s%%", *tv ); */
#endif
		    if (!(freopen(buf, "w", stdout))) {
                        perror(buf);
                        shutdown(EXIT_BAD);
                    }
                    **tv = Spice_OptChar;
                } else {
                    fprintf(cp_err, usage, av[0]);
                    shutdown(EXIT_BAD);
                }
                break;

	    case 'c':   /* Circuit file. */
	    case 'C':
                if (tc > 1) {
                    tc--;
                    tv++;
		    if (!(circuit_file = fopen(*tv, "r"))) {
			perror(*tv);
			shutdown(EXIT_BAD);
		    }
		    istty = false;
                    **tv = Spice_OptChar;
                } else {
                    fprintf(cp_err, usage, av[0]);
                    shutdown(EXIT_BAD);
                }
                break;

#ifdef PARALLEL_ARCH
	    case 'm':   /* '-master' option from "parallel". */
                if (tc > 6) {
		    for (i =0; i < 6; i++) {
		      tc--;
		      tv++;
		      **tv = Spice_OptChar;
		    }
                } else {
                    fprintf(cp_err, usage, 
                        cp_program);
                    shutdown(EXIT_BAD);
                }
                break;
#endif /* PARALLEL_ARCH */

	    default:
                fprintf(cp_err, "Error: bad option %s\n", *tv);
                fprintf(cp_err, usage, cp_program);
                shutdown(EXIT_BAD);
            }
    }

#ifdef SIMULATOR
    if_getparam = spif_getparam;
#else
    if_getparam = nutif_getparam;
#endif



    if ((!iflag && !istty) || ft_servermode)
        ft_batchmode = true;
    if ((iflag && !istty) || qflag)
        cp_nocc = true;
    if (ft_servermode)
        readinit = false;
    if (!istty || ft_batchmode)
        out_moremode = false;

    /* Would like to do this later, but cpinit evals commands */
    init_rlimits( );

    /* Have to initialize cp now. */
    ft_cpinit();

    /* To catch interrupts during .spiceinit... */
    if (setjmp(jbuf) == 1) {
        fprintf(cp_err, "Warning: error executing .spiceinit.\n");
        if (!ft_batchmode)
            goto bot;
    }
 
    /* Set up (void) signal handling */
    if (!ft_batchmode) {
        (void) signal(SIGINT, ft_sigintr);
        (void) signal(SIGFPE, sigfloat);
#    ifdef SIGTSTP
        (void) signal(SIGTSTP, sigstop);
#    endif
    }
    /* Set up (void) signal handling for fatal errors. */
    (void) signal(SIGILL, sigill);

#    ifdef SIGBUS
    (void) signal(SIGBUS, sigbus);
#    endif
#    ifdef SIGSEGV
    (void) signal(SIGSEGV, sigsegv);
#    endif
#    ifdef SIGSYS
    (void) signal(SIGSYS, sig_sys);
#    endif


#  ifdef HAVE_PWD_H
    if (readinit) {
        /* Try to source either .spiceinit or ~/.spiceinit. */
        if (access(".spiceinit", 0) == 0)
            inp_source(".spiceinit");
        else {
            pw = getpwuid(getuid());
            (void) strcpy(buf, pw->pw_dir);
            (void) strcat(buf, "/.spiceinit");
            if (access(buf, 0) == 0)
                inp_source(buf);
        }
    }
#  else /* ~ HAVE_PWD_H */
    /* Try to source the file "spice.rc" in the current directory.  */
    if (readinit) {
        if ((fp = fopen("spice.rc", "r")) != NULL) {
            (void) fclose(fp);
            inp_source("spice.rc");
        }
    }
#  endif /* ~ HAVE_PWD_H */

    if (cmd_line_term) {
	*cmd_line_term = term_1stch; /* XXX oh, gross!
				      *	First char got squashed scanning the
				      * command line
				      */
	cp_vset("term", VT_STRING, cmd_line_term);
    }

    if (!ft_batchmode) {
	com_version(NULL);
        DevInit( );
	if (News_File && *News_File) {
	    fp = fopen(cp_tildexpand(News_File), "r");
	    if (fp) {
		while (fgets(buf, BSIZE_SP, fp))
		    fputs(buf, stdout);
		(void) fclose(fp);
	    }
	}
    }


bot:

    /* Pass 2 -- get the filenames. If we are spice, then this means
     * build a circuit for this file. If this is in server mode, don't
     * process any of these args.
     */

    if (setjmp(jbuf) == 1)
        goto evl;


    cp_interactive = false;
    err = 0;

#ifdef SIMULATOR
    if (!ft_servermode && !ft_nutmeg) {
        FILE *file = NULL, *tp = NULL;
        char *tempfile = NULL, buf[BSIZE_SP], *smktemp();

        for (tv = av + 1, i = 0; *tv; tv++)
            if (**tv != Spice_OptChar)
                i++;
        if (i == 1) {
            for (tv = av + 1, i = 0; *tv; tv++)
                if (**tv != Spice_OptChar)
                    break;
            if (!(file = fopen(*tv, "r"))) {
                perror(*tv);
                i = 0;
		if (ft_batchmode)
		    shutdown(EXIT_BAD);
            }
        } else if (i) {
            tempfile = smktemp("sp");
            if (!(file = fopen(tempfile, "w+"))) {
                perror(tempfile);
                shutdown(EXIT_BAD);
            }
            for (tv = av + 1, i = 0; *tv; tv++)
                if (**tv != Spice_OptChar) {
                    if (!(tp = fopen(*tv, "r"))) {
                        perror(*tv);
			err = 1;
                        break;
                    }
                    while ((i = fread(buf, 1, BSIZE_SP, tp)) > 0)
                        (void) fwrite(buf, i, 1, file);
                    (void) fclose(tp);
                }
            (void) fseek(file, (long) 0, 0);
        }
        if (file && (!err || !ft_batchmode)) {
            inp_spsource(file, false, tempfile ? (char *) NULL : *tv);
            gotone = true;
        }
	if (tempfile)
	    (void) unlink(tempfile);
	if (ft_batchmode && err)
	    shutdown(EXIT_BAD);
    }
#endif


    if (ft_nutmeg && gdata) {
        /* Read in the rawfiles */
        for (av++; *av; av++)
            if (**av != Spice_OptChar) {
                ft_loadfile(*av);
                gotone = true;
            }
        if (!gotone)
            ft_loadfile(ft_rawfile);
    }


#ifdef SIMULATOR
    if (!gotone && ft_batchmode && !ft_nutmeg)
        inp_spsource(circuit_file, false, (char *) NULL);


evl:
    if (ft_batchmode) {
        /* If we get back here in batch mode then something is
         * wrong, so exit.
         */
        bool st = false;

        (void) setjmp(jbuf);

        if (st == true) {
            shutdown(EXIT_BAD);
	}
        st = true;
        if (ft_servermode) {
            if (ft_curckt == NULL) {
                fprintf(cp_err, "Error: no circuit loaded!\n");
                shutdown(EXIT_BAD);
            }
            if (ft_dorun(""))
		shutdown(EXIT_BAD);
            shutdown(EXIT_NORMAL);
        }
        /* If -r is specified, then we don't bother with the dot
         * cards. Otherwise, we use wrd_run, but we are careful
         * not to save too much.
         */
        cp_interactive = false;
        if (rflag) {
	    ft_dotsaves();
	    error2 = ft_dorun(ft_rawfile);
	    if (ft_cktcoms(true) || error2)
		shutdown(EXIT_BAD);
        } else {
            if (ft_savedotargs()) {
		error2 = ft_dorun(NULL);
		if (ft_cktcoms(false) || error2)
		    shutdown(EXIT_BAD);
	    } else {
		fprintf(stderr,
		    "Note: No \".plot\", \".print\", or \".fourier\" lines; no simulations run\n");
		shutdown(EXIT_BAD);
	    }
        }
    } else {
        (void) setjmp(jbuf);
        cp_interactive = true;
	while (cp_evloop((char *) NULL) == 1) ;
    }


#else

evl:
    /* Nutmeg "main" */
    (void) setjmp(jbuf);
    cp_interactive = true;
    while (cp_evloop((char *) NULL) == 1) ;

#endif

    shutdown(EXIT_NORMAL);
    return EXIT_NORMAL;
}

/* Shutdown gracefully. */
int
shutdown(exitval)
int exitval;
{
#ifdef PARALLEL_ARCH
    if (exitval == EXIT_BAD) {
	Error("Fatal error in SPICE", -1);
    } else {
	PEND_();
    }
#endif /* PARALLEL_ARCH */
    exit(exitval);
}

/* allocate space for global constants in 'CONST.h' */

double CONSTroot2;
double CONSTvt0;
double CONSTKoverQ;
double CONSTe;
IFfrontEnd *SPfrontEnd = NULL;

int
SPIinit(frtEnd,description)
    IFfrontEnd *frtEnd;
    IFsimulator **description;
{
    SPfrontEnd = frtEnd;
    *description = &SIMinfo;
    CONSTroot2 = sqrt(2.);
    CONSTvt0 = CONSTboltz * (27 /* deg c */ + CONSTCtoK ) / CHARGE;
    CONSTKoverQ = CONSTboltz / CHARGE;
    CONSTe = exp((double)1.0);
    return(OK);
}

/* XXX SIMinit and SPIinit ?? */

int SIMinit(frontEnd,simulator)
    IFfrontEnd *frontEnd;
    IFsimulator **simulator;
{
    return(SPIinit(frontEnd,simulator));
}

