/*************
 * Definitions header file
 * 1999 E. Rouat
 ************/

/* 
 * This file will contain all #defines needed
 * by ngspice code (in construction)
 * It should only #define numeric constants, not macros.
 */


#ifndef _DEFINES_H_
#define _DEFINES_H_


#ifndef	M_PI
#  define M_PI       3.14159265358979323846
#endif
#ifndef	M_E
#  define M_E  	     2.7182818284590452354
#endif
#ifndef	M_LOG2E
#  define M_LOG2E    1.4426950408889634074
#endif
#ifndef	M_LOG10E
#  define M_LOG10E   0.43429448190325182765
#endif


/*
 *  IEEE Floating point
 */

#define MAX_EXP_ARG	709.0

#ifndef DBL_EPSILON
# define DBL_EPSILON	8.9e-15
#endif
#ifndef DBL_MAX
# define DBL_MAX	1.79769313486231e+308
#endif
#ifndef DBL_MIN
# define DBL_MIN	2.22507385850721e-308
#endif
#ifndef SHRT_MAX
# define SHRT_MAX	32766
#endif
#ifndef INT_MAX
# define INT_MAX	2147483646
#endif
#ifndef LONG_MAX
# define LONG_MAX	2147483646
#endif

#define MAXPOSINT	INT_MAX

/* 
 * Physical constants (const.h)
 */

#define CHARGE (1.6021918e-19)
#define CONSTCtoK (273.15)
#define CONSTboltz (1.3806226e-23)
#define REFTEMP 300.15 /* 27 degrees C */



#define DIR_PATHSEP	"/"
#define DIR_TERM	'/'
#define DIR_CWD		"."

#define TEMPFORMAT	"/tmp/%s%d"
#define SYSTEM_PLOT5LPR	"lpr -P%s -g %s"
#define SYSTEM_PSLPR	"lpr -P%s %s"
#define SYSTEM_MAIL	"Mail -s \"%s (%s) Bug Report\" %s"


/*
 *  #define-s that are always on
 */

#define CAPZEROBYPASS
#define NEWCONV
/* #define CAPBYPASS	Internal use only */


/* On Unix the following should always be true, so they should jump out */

#define HAS_ASCII
#define HAS_TTY_
#define HAS_TIME_
#define HAS_RLIMIT_



#ifndef SIGNAL_FUNCTION
#  define SIGNAL_FUNCTION RETSIGTYPE (*)( )
#endif

#define BSIZE_SP      512


#define EXIT_NORMAL 0
#define EXIT_BAD    1

#define TRUE 1
#define FALSE 0



/* common error message descriptions */

#define E_PAUSE -1      /* pausing on demand */

#define OK 0

#define E_PANIC 1       /* vague internal error for "can't get here" cases */
#define E_EXISTS 2      /* warning/error - attempt to create duplicate */
                        /* instance or model. Old one reused instead */
#define E_NODEV 3       /* attempt to modify a non-existant instance */
#define E_NOMOD 4       /* attempt to modify a non-existant model */
#define E_NOANAL 5      /* attempt to modify a non-existant analysis */
#define E_NOTERM 6      /* attempt to bind to a non-existant terminal */
#define E_BADPARM 7     /* attempt to specify a non-existant parameter */
#define E_NOMEM 8       /* insufficient memory available - VERY FATAL */
#define E_NODECON 9     /* warning/error - node already connected, old */
                        /* connection replaced */
#define E_UNSUPP 10     /* the specified operation is unsupported by the */
                        /* simulator */
#define E_PARMVAL 11    /* the parameter value specified is illegal */
#define E_NOTEMPTY 12   /* deleted still referenced item. */
#define E_NOCHANGE 13   /* simulator can't tolerate any more topology changes */
#define E_NOTFOUND 14   /* simulator can't find something it was looking for */
#define E_BAD_DOMAIN 15 /* output interface begin/end domain calls mismatched */


#define E_PRIVATE 100   /* messages above this number are private to */
                        /* the simulator and MUST be accompanied by */
                        /* a proper setting of errMsg */
                        /* this constant should be added to all such messages */
                        /* to ensure error free operation if it must be */
                        /* changed in the future */

extern char *errMsg;    /* descriptive message about what went wrong */
                        /* MUST be malloc()'d - front end will free() */
                        /* this should be a detailed message,and is assumed */
                        /* malloc()'d so that you will feel free to add */
                        /* lots of descriptive information with sprintf*/

extern char *errRtn;    /* name of the routine declaring error */
                        /* should not be malloc()'d, will not be free()'d */
                        /* This should be a simple constant in your routine */
                        /* and thus can be set correctly even if we run out */
                        /* of memory */


/*
 * definitions for error codes returned by SPICE3 routines.
 */

#define E_INTERN E_PANIC
#define E_BADMATRIX (E_PRIVATE+1)/* ill-formed matrix can't be decomposed */
#define E_SINGULAR (E_PRIVATE+2) /* matrix is singular */
#define E_ITERLIM (E_PRIVATE+3)  /* iteration limit reached,operation aborted */
#define E_ORDER (E_PRIVATE+4)    /* integration order not supported */
#define E_METHOD (E_PRIVATE+5)   /* integration method not supported */
#define E_TIMESTEP (E_PRIVATE+6) /* timestep too small */
#define E_XMISSIONLINE (E_PRIVATE+7)    /* transmission line in pz analysis */
#define E_MAGEXCEEDED (E_PRIVATE+8) /* pole-zero magnitude too large */
#define E_SHORT (E_PRIVATE+9)   /* pole-zero input or output shorted */
#define E_INISOUT (E_PRIVATE+10)    /* pole-zero input is output */
#define E_ASKCURRENT (E_PRIVATE+11) /* ac currents cannot be ASKed */
#define E_ASKPOWER (E_PRIVATE+12)   /* ac powers cannot be ASKed */
#define E_NODUNDEF (E_PRIVATE+13) /* node not defined in noise anal */
#define E_NOACINPUT (E_PRIVATE+14) /* no ac input src specified for noise */
#define E_NOF2SRC (E_PRIVATE+15) /* no source at F2 for IM disto analysis */
#define E_NODISTO (E_PRIVATE+16) /* no distortion analysis - NODISTO defined */
#define E_NONOISE (E_PRIVATE+17) /* no noise analysis - NONOISE defined */
#ifdef PARALLEL_ARCH
#define E_MULTIERR (E_PRIVATE+18) /* multiple errors from diff. processes */
#endif /* PARALLEL_ARCH */



/* ckt */


#define MAXNUMDEVS 32     /* Max number of possible devices; */
extern int DEVmaxnum;     /* Not sure if still used */
#define MAXNUMDEVNODES 4  /* Max No. of nodes per device */
                          /* Need to change for SOI devs ? */

/* defines for node parameters */
#define PARM_NS 1
#define PARM_IC 2
#define PARM_NODETYPE 3


#define SP_VOLTAGE 3
#define SP_CURRENT 4
#define NODE_VOLTAGE SP_VOLTAGE
#define NODE_CURRENT SP_CURRENT

  /* Some shortcut for CKTstates */
#define CKTstate0 CKTstates[0]
#define CKTstate1 CKTstates[1]
#define CKTstate2 CKTstates[2]
#define CKTstate3 CKTstates[3]
#define CKTstate4 CKTstates[4]
#define CKTstate5 CKTstates[5]
#define CKTstate6 CKTstates[6]
#define CKTstate7 CKTstates[7]

/*
 *  symbolic constants for CKTniState
 *      Note that they are bitwise disjoint
 *  What is their meaning ????
 */

#define NISHOULDREORDER 0x1
#define NIREORDERED 0x2
#define NIUNINITIALIZED 0x4
#define NIACSHOULDREORDER 0x10
#define NIACREORDERED 0x20
#define NIACUNINITIALIZED 0x40
#define NIDIDPREORDER 0x100
#define NIPZSHOULDREORDER 0x200

  /* This define should be somewhere else ??? */
#define NODENAME(ckt,nodenum) CKTnodName(ckt,nodenum)


/* defines for CKTmode */

/* old 'mode' parameters */
#define MODE 0x3
#define MODETRAN 0x1
#define MODEAC 0x2

/* old 'modedc' parameters */
#define MODEDC 0x70
#define MODEDCOP 0x10
#define MODETRANOP 0x20
#define MODEDCTRANCURVE 0x40

/* old 'initf' parameters */
#define INITF 0x3f00
#define MODEINITFLOAT 0x100
#define MODEINITJCT 0x200
#define MODEINITFIX 0x400
#define MODEINITSMSIG 0x800
#define MODEINITTRAN 0x1000
#define MODEINITPRED 0x2000

/* old 'nosolv' paramater */
#define MODEUIC 0x10000l


/* fteconst.h */

/* Vector types. */


#define SV_NOTYPE   0
#define SV_TIME  1
#define SV_FREQUENCY    2
#define SV_VOLTAGE  3
#define SV_CURRENT  4
#define SV_OUTPUT_N_DENS 5
#define SV_OUTPUT_NOISE  6
#define SV_INPUT_N_DENS 7
#define SV_INPUT_NOISE  8
#define SV_POLE     9
#define SV_ZERO     10
#define SV_SPARAM   11



/* Dvec flags. */

#define VF_REAL      (1 << 0) /* The data is real. */
#define VF_COMPLEX   (1 << 1) /* The data is complex. */
#define VF_ACCUM     (1 << 2) /* writedata should save this vector. */
#define VF_PLOT      (1 << 3) /* writedata should incrementally plot it. */
#define VF_PRINT     (1 << 4) /* writedata should print this vector. */
#define VF_MINGIVEN  (1 << 5) /* The v_minsignal value is valid. */
#define VF_MAXGIVEN  (1 << 6) /* The v_maxsignal value is valid. */
#define VF_PERMANENT (1 << 7) /* Don't garbage collect this vector. */

/* Grid types. */

/*
#define GRID_NONE   0
#define GRID_LIN    1
#define GRID_LOGLOG 2
#define GRID_XLOG   3
#define GRID_YLOG   4
#define GRID_POLAR  5
#define GRID_SMITH  6
*/

/* SMITHGRID is only a smith grid, SMITH transforms the data */
typedef enum {
    GRID_NONE = 0, GRID_LIN = 1, GRID_LOGLOG = 2, GRID_XLOG = 3,
    GRID_YLOG = 4, GRID_POLAR = 5, GRID_SMITH = 6, GRID_SMITHGRID = 7
} GRIDTYPE;

/* Plot types. */

/*
#define PLOT_LIN    0
#define PLOT_COMB   1
#define PLOT_POINT  2
*/

typedef enum {
    PLOT_LIN = 0, PLOT_COMB = 1, PLOT_POINT = 2
} PLOTTYPE;

/* The types for command completion keywords. Note that these constants
 * are built into cmdtab.c, so DON'T change them unless you want to
 * change all of the bitmasks in cp_coms.
 * Note that this is spice- and nutmeg- dependent.
 */

#define CT_FILENAME     0
#define CT_CKTNAMES     2
#define CT_COMMANDS     3
#define CT_DBNUMS       4
#define CT_DEVNAMES     5
#define CT_LISTINGARGS  6
#define CT_NODENAMES    7
#define CT_PLOT         8
#define CT_PLOTKEYWORDS 9
#define CT_RUSEARGS     10
#define CT_STOPARGS     11
#define CT_UDFUNCS      12
#define CT_VARIABLES    13
#define CT_VECTOR       14
#define CT_TYPENAMES    16





#endif /* _DEFINES_H_ */

