/*
 * Main header file for NGSPICE 
 * This header file includes things from former misc.h and util.h
 */

#ifndef SPICE_H_INCLUDED
#define SPICE_H_INCLUDED 

/*
 *	Portability, global externs, kitchen sink (yuk).
 *	In future releases this will include things from misc.h and util.h,
 *	   which duplicate each other in places
 */

#include <math.h>

#ifndef	M_PI
#  define M_PI		3.14159265358979323846
#endif
#ifndef	M_E
#  define M_E  	   2.7182818284590452354
#endif
#ifndef	M_LOG2E
#  define M_LOG2E		1.4426950408889634074
#endif
#ifndef	M_LOG10E
#  define M_LOG10E        0.43429448190325182765
#endif


#ifndef	PI
#  define PI		3.14159265358979323846
#endif
#ifndef	M_E
#  define M_E  	   2.7182818284590452354
#endif
#ifndef	LOG2E
#  define LOG2E		1.4426950408889634074
#endif
#ifndef	LOG10E
#  define LOG10E        0.43429448190325182765
#endif


#include <config.h>
#include "hw_ieee.h"
#include "missing_math.h"
#include "arch.h"

#define	NUMELEMS(ARRAY)	(sizeof(ARRAY)/sizeof(*ARRAY))


#define DIR_PATHSEP	"/"
#define DIR_TERM	'/'
#define DIR_CWD		"."

#define TEMPFORMAT	"/tmp/%s%d"
#define SYSTEM_PLOT5LPR	"lpr -P%s -g %s"
#define SYSTEM_PSLPR	"lpr -P%s %s"
#define SYSTEM_MAIL	"Mail -s \"%s (%s) Bug Report\" %s"

/* wasn't too sure about removing the next one..*/

#define HAS_ASCII

/*
 *  #define-s that are always on
 */

#define CAPZEROBYPASS
#define NEWCONV
/* #define CAPBYPASS	Internal use only */


/* On Unix we always have tty capability so the next def could jump out */

# define HAS_TTY_

/* same here */

# define HAS_TIME_
# define HAS_RLIMIT_



#ifndef SIGNAL_FUNCTION
#  define SIGNAL_FUNCTION RETSIGTYPE (*)( )
#endif




extern char *Spice_Exec_Dir;
extern char *Spice_Lib_Dir;
extern char Spice_OptChar;
extern char *Def_Editor;
extern char *Bug_Addr;
extern int AsciiRawFile;
extern char *Spice_Host;
extern char *Spiced_Log; 

extern char Spice_Notice[ ];
extern char Spice_Version[ ];
extern char Spice_Build_Date[ ];

extern char *News_File;
extern char *Default_MFB_Cap;
extern char *Spice_Path;
extern char *Help_Path;
extern char *Lib_Path;



/* misc.h */


#define BSIZE_SP      512

#ifdef HAS_EXIT1
#  define EXIT_NORMAL 1
#  define EXIT_BAD    0
#else
#  define EXIT_NORMAL 0
#  define EXIT_BAD    1
#endif

#ifdef HAVE_CTYPE_H
#  ifndef isalpha
#    include <ctype.h>
#  endif
#endif

#define eq(a,b)  (!strcmp((a), (b)))
#define eqc(a,b)  (cieq((a), (b)))
#define isalphanum(c)   (isalpha(c) || isdigit(c))
#define hexnum(c) ((((c) >= '0') && ((c) <= '9')) ? ((c) - '0') : ((((c) >= \
        'a') && ((c) <= 'f')) ? ((c) - 'a' + 10) : ((((c) >= 'A') && \
        ((c) <= 'F')) ? ((c) - 'A' + 10) : 0)))


#ifndef STDC_HEADERS
#  include <strings.h>
#else
#  include <string.h>
#endif



#define tfree(x)	(txfree(x), x = 0)

#define	alloc(TYPE)	((TYPE *) tmalloc(sizeof(TYPE)))



#include <sys/types.h>

/* Externs from libc */

#ifdef STDC_HEADERS

#  ifndef _STDLIB_INCLUDED
#    define _STDLIB_INCLUDED
#    include <stdlib.h>
#  endif



#else

extern char *calloc();
extern char *malloc();
extern char *realloc();
extern char *getenv();
extern int errno;
extern char *sys_errlist[];
extern char *getenv();
extern char *getwd();
extern int rand();
extern int srand();
extern int atoi();
extern int kill();
extern pid_t getpid();
extern int qsort();
#  ifdef notdef
extern void exit();
#  endif

#  ifdef HAVE_GETCWD
extern char *getcwd( );
#  endif



#ifndef HAVE_BCOPY 
#  ifndef bzero
extern int bzero();
#  endif
#  ifndef bcopy
extern void bcopy();
#  endif
#endif /*HAVE_BCOPY*/

#ifndef HAVE_INDEX
#   ifndef HAVE_STRCHR
extern char *strchr();
extern char *strrchr();
#   else
extern char *rindex();
extern char *index();
#    endif
#  endif


#endif	/* else STDC_HEADERS  */

#ifdef HAVE_STRCHR
#  ifndef HAVE_INDEX
#      define	index	strchr
#      define	rindex	strrchr
#  endif
#endif

/* Functions declarations from src/lib/misc/[].c  */

extern void *tmalloc(size_t num);
extern void *trealloc(void *str, size_t num);
extern void txfree(void *ptr);

extern char *gettok(char **s);
extern void appendc(char *s, char c);
extern int scannum(char *str);
extern int ciprefix(register char *p, register char *s);
extern int cieq(register char *p, register char *s);
extern void strtolower(char *str);
extern char *tilde_expand(char *string);

extern char *smktemp(char *id);

extern char *copy();
extern int prefix();
extern int substring();
extern void cp_printword();

extern char *datestring();
extern double seconds(void);



extern void perror();


#ifdef HAS_TIME_
#  ifdef HAVE_GETTIMEOFDAY
/* extern char *timezone(); */  /* never used ? (ER) */
#  endif
extern char *asctime();
extern struct tm *localtime();
#endif

#ifdef HAS_RLIMIT_
extern char *sbrk();
#endif

#define false 0
#define true 1




/* util.h */


/* #define MALLOC(x) calloc(1,(unsigned)(x)) */
#define MALLOC(x) tmalloc((unsigned)(x))
#define FREE(x) {if (x) {free((char *)(x));(x) = 0;}}
#define REALLOC(x,y) trealloc((char *)(x),(unsigned)(y))
#define ZERO(PTR,TYPE)	(bzero((PTR),sizeof(TYPE)))

#ifdef STDC_HEADERS
#ifndef _STDLIB_INCLUDED
#define _STDLIB_INCLUDED
#include <stdlib.h>
#endif
#else
extern char *malloc();
extern char *calloc();
extern char *realloc();
extern void free();
#endif

#define TRUE 1
#define FALSE 0

#ifdef DEBUG
#define DEBUGMSG(textargs) printf(textargs)
#else
#define DEBUGMSG(testargs) 
#endif

#ifdef HAS_NOINLINE
#define FABS(a) fabs(a)
double fabs();
#else
#define FABS(a) ( ((a)<0) ? -(a) : (a) )
#endif

/* XXX Move these into the above ifdef someday */
#define MIN(a,b) ((a) < (b) ? (a) : (b))
#define MAX(a,b) ((a) > (b) ? (a) : (b))
#define SIGN(a,b) ( b >= 0 ? (a >= 0 ? a : - a) : (a >= 0 ? - a : a))

#define ABORT() fflush(stderr);fflush(stdout);abort();

#define ERROR(CODE,MESSAGE)	{					      \
	errMsg = MALLOC(strlen(MESSAGE) + 1);				      \
	strcpy(errMsg, (MESSAGE));					      \
	return (CODE);							      \
	}

#define	NEW(TYPE)	((TYPE *) MALLOC(sizeof(TYPE)))
#define	NEWN(TYPE,COUNT) ((TYPE *) MALLOC(sizeof(TYPE) * (COUNT)))


#define	R_NORM(A,B) {							      \
	if ((A) == 0.0) {						      \
	    (B) = 0;							      \
	} else {							      \
	    while (FABS(A) > 1.0) {					      \
		(B) += 1;						      \
		(A) /= 2.0;						      \
	    }								      \
	    while (FABS(A) < 0.5) {					      \
		(B) -= 1;						      \
		(A) *= 2.0;						      \
	    }								      \
	}								      \
    }


#endif /* SPICE_H_INCLUDED */
